% Data from working paper by Thomas Coleman, Yuying Li, and Arun Verma
% "Reconstructing the Unknown Local Volatility Function"
% By Fabrice Douglas Rouah

clc; clear;

% Black Scholes call price
BSCall = @(s,K,r,q,v,T) s*exp(-q*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) - v*sqrt(T));

%% Call option data
% Spot price, rf rate, dividend yield
S0 = 590;
r = 0.06;
q = 0.0262;

% Implied volatility.  Rows are maturities, so transpose.
IV = [...
    190	168	133	113	102	97	120	142	169	200;
    177	155	138	125	109	103	100	114	130	150;
    172	157	144	133	118	104	100	101	108	124;
    171	159	149	137	127	113	106	103	100	110;
    171	159	150	138	128	115	107	103	99	108;
    169	160	151	142	133	124	119	113	107	102;
    169	161	153	145	137	130	126	119	115	111;
    168	161	155	149	143	137	133	128	124	123;
    168	162	157	152	148	143	139	135	130	128;
    168	164	159	154	151	148	144	140	139	132]./1000;
IV = IV';

% Strikes
K = [501.50 531.00 560.50 590.00 619.50 649.00 678.50 708.00 767.00 826.00];
NK = length(K);

% Maturities T
T = [0.175 0.425 0.695 0.94 1 1.5 2 3 4 5];
NT = length(T);


%% Obtain the local volatility from the implied vol
% Increments for the finite difference approximations to derivatives
dK = 1;
dT = 0.001;

% Local vol from implied vol
LV = getLVfromIV(S0,K,T,IV,dK,dT,r,q)

% Interpolate the local vol surface
NKi = 60;
NTi = 50;
dK = (K(end)-K(1))/(NKi-1);
dT = (T(end)-T(1))/(NTi-1);
Ki = K(1):dK:K(end);
Ti = T(1):dT:T(end);
LVi = interp2(K,T,LV,Ki,Ti','cubic');


%% Plot the IV and rough LV surfaces
subplot(1,2,1)
surf(Ki,Ti,LVi)
xlabel('Strike')
ylabel('Maturity')
hold on
mesh(K,T,IV,'Marker','o','MarkerFaceColor','k')
alpha(0.5)
title('Implied Vol (dots) and Rough Local Vol (surface)')
hold off


%% Fit a quadratic surface for LV
Z = LV(:);
[x y] = meshgrid(K,T);
X = x(:);
Y = y(:);
R = regstats(Z,[X Y],'quadratic');
b = R.beta;
start = b;
[b feval] = fminsearch(@(p) findQuadParam(p,K,T,LV), start);

for k=1:length(Ki)
    for t=1:length(Ti)
        LVq(k,t) = b(1) + b(2)*Ki(k) + b(3)*Ti(t) + b(4)*Ki(k)*Ti(t) + b(5)*Ki(k)^2 + b(6)*Ti(t)^2;
    end
end

%% Plot the IV and smoothed quadratic LV surface
subplot(1,2,2)
surf(Ki,Ti,LVq')
xlabel('Strike')
ylabel('Maturity')
zlim([0 .6]);
hold on
mesh(K,T,IV,'Marker','o','MarkerFaceColor','k')
alpha(0.5)
title('Implied Vol (dots) and Quadratic Local Vol (surface)')
hold off
