function LV = getLVfromIV(S0,K,T,IV,dK,dT,r,q)

% S0 = spot price
% K = vector of strikes
% T = vector of maturities
% IV = matrix of implied vols (rows are maturities, cols are strikes
% dK,dT = desired increment for finite differences
% r,q  = risk free rate, dividend yield

NK = length(K);
NT = length(T);

% Expansion of strike and maturity domain for interpolation grid
% Append increments of (dK) and (2*dK) for each strike
KI = [];
for k=1:NK
    KI = [KI, K(k)-2*dK, K(k)-dK, K(k), K(k)+dK, K(k)+2*dK];
end

% Append increments of (dT) and (2*dT) for each maturity
TI = [];
for t=1:NT
    TI = [TI, T(t)-2*dT, T(t)-dT, T(t), T(t)+dT, T(t)+2*dT];
end

% Number of strikes and maturities
NKI = length(KI);
NTI = length(TI);

%% Interpolate implied vol
% This is used to obtain partial derivatives of Implied Vol with
% respect to T and K
IVI = interp2(T,K,IV,TI',KI,'spline');

%% Partial derivatives of implied vol wrt strike
DerK = zeros(NKI,NTI);
for k=2:NKI-1
    for t=2:NTI-1
        DerK(k,t) = (IVI(k+1,t) - IVI(k-1,t))/2/dK;
    end;
end
% Retain only every 5th observation, corresponding to original 
dVdK = DerK([3:5:NKI], [3:5:NTI]);

%% Partial derivatives of implied vol wrt maturity
DerT = zeros(NKI,NTI);
for k=2:NKI-1
    for t=2:NTI-1
        DerT(k,t) = (IVI(k,t+1) - IVI(k,t-1))/2/dT;
    end
end
% Retain only every 5th observation, corresponding to original 
dVdT = DerT([3:5:NKI], [3:5:NTI]);

%% Second partial derivatives of implied vol wrt strike
DerK2 = zeros(NKI,NTI);
for k=3:NKI-2
    for t=3:NTI-2
        DerK2(k,t) = (IVI(k+1,t) - 2*IVI(k,t) + IVI(k-1,t))/(dK^2);
    end
end
% Retain only every 5th observation, corresponding to original 
dV2dK2 = DerK2([3:5:NKI], [3:5:NTI]);

%% Local Variance from Implied Volatility
for t=1:NT
    for k=1:NK
        Num = 2*IV(k,t)*dVdT(k,t)*T(t) + IV(k,t)^2 + ...
              2*IV(k,t)*(r-q)*K(k)*T(t)*dVdK(k,t);
         d1 = (log(S0/K(k)) + (r-q-IV(k,t)^2/2)*T(t)) / IV(k,t) / sqrt(T(t));
        Den = (1+ K(k)*d1*dVdK(k,t)*sqrt(T(t)))^2 + K(k)^2*T(t)*IV(k,t)*(dV2dK2(k,t) - d1*dVdK(k,t)^2*sqrt(T(t)));
        LocalVar(k,t) = Num/Den;
    end
end

%% Get rid of negative values in Local Variance using quadratic function
for t=1:NT;
    % Extract each maturity
    Y = LocalVar(:,t);
    % Identify maturities with negative Local Variances
    if sum(Y<=0) >= 1
        I = find(Y>0);
        % Parameter estimates on positive Local variances
        b = regstats(Y(I),K(I),'purequadratic');
        % Replace negative local variances with fitted values
        Ic = find(Y<=0);
        for i=1:length(Ic);
            Y(Ic(i)) = b.beta(1) + b.beta(2)*K(Ic(i)) + b.beta(3)*K(Ic(i))^2;
            Y(Ic(i)) = max(0.01,Y(Ic(i)));
        end
        % Replace the column with fitted values
        LocalVar(:,t) = Y;
    end
end


%% Obtain Local volatility as the square root of local variance
LV = sqrt(LocalVar);
